///	@file	strutil.cpp
///	@brief	strutil.cpp
//#include <sstream>
#include <iomanip>
#include <math.h>
#include <ctype.h>

#include "strutil.h"
#include "mlslocale.h"

using namespace std;
using namespace MLS;

///	@brief	글를 소문자로 만들다.
///	@return	소문자로 만들어질 글
string strutil::tolower(
	const string &p			///< 소문자로 만들 글
	)
{
	string r = p;
	std::transform(r.begin(), r.end(), r.begin(), (int(*)(int)) std::tolower);
	return r;
}

///	@brief	글를 대문자로 만들다.
///	@return	대문자로 만들어질 글
string strutil::toupper(
	const string &p			///< 대문자로 만들 글
	)
{
	string r = p;
	std::transform(r.begin(), r.end(), r.begin(), (int(*)(int)) std::toupper);
	return r;
}

/// @brief	필요 없는 문자를 제거한다.
///	@return	공백, 엔터등을 제거한 문자열 알맹이
string strutil::chop(
	const string &p			///< 문자열 
	)
{
	string::size_type f = p.find_first_not_of(" \n\r");
	string::size_type r = p.find_last_not_of(" \n\r");
	
	if (f == string::npos || r== string::npos) return "";
	
	return p.substr(f, r-f+1);
}

/// @brief	p문자열에서 위치 f, r 사이의 문자열을 반환한다.
string strutil::getbetween(
	const string &p,			///< 문자열
	const char f,				///< 반환할 문자의 시작 위치
	const char r				///< 반환할 문자의 끝위치
	)
{
	string::size_type s = p.find_first_of(f);
	if (s == string::npos) return "";
	string::size_type s2 = p.find_first_of(r, s+1);
	if (s2 == string::npos) return "";	
	
	return p.substr(s+1 , s2-s-1);	
}

///	@brief	해당되는 글자를 다른 글자로 바꾼다.
///	@return	바꾸어진 글
string strutil::replace(
	const string &p,		///< 글자를 바꿀 대상 전체
	const char*find,		///< 바꿀 대상 문자
	const char*replace		///< 바꿀 문자
	)
{
	//  01234567890123456789012
	// "the great great bassist" "great" "brother"	
	string arg;
	
	string::size_type t = 0;
	string::size_type t2= p.find(find);
	
	string::size_type len = strlen(find);
	
	while(t2 != string::npos)
	{
		arg += p.substr(t, t2-t);
		arg += replace;
		
		t = t2 + len;
		t2 = p.find(find, t);
	}
	
	arg += p.substr(t);
	
	return arg;
}

///	@brief	숫자를 문자로 바꾸어준다.
///	@return	문자화 된 숫자
string strutil::toregular(
	unsigned int number		///< number	문자로 바꿀 숫자
	)
{
	unsigned int n=1, p=1;
	string str;	
	str.reserve(10);
			
	while(number/10 >= p) {p*=10; n+=1;}
	
	while (n)
	{
		str += '0' + number / p;
		
		number %= p;
		p /= 10;
		n--;
		
		if (n!=0 && n%3==0) str += ',';	
	}
	
	return str;
}

/// @brief	string에 slash를 추가하는 함수
///
/// 파일이름중에서 특수문자가(예 공백이나 () & *등..을 '\ ' '\ ('로 바꾸어
/// 주는함수
///
/// @return 	shlash가 추가된 글.
string strutil::addslash(
	const string &p			///< 문자열.
	)
{
	//  t
	// 01234
	// jones
	string arg;
	
	string::size_type t  = 0,
					  t2 = p.find_first_of(" ()&*?$'\"\\\0");
	
	while(t2 != string::npos)
	{
		arg.append(p, t, t2-t);
		arg += '\\';
		arg += p[t2];		
		t = t2 + 1;
		t2 = p.find_first_of(" ()&*?$'\"\\\0", t);
	}	
	arg.append(p, t, p.length()-t);
	
	return arg;			
}

/// @brief  한글이 깨지지 않게 잘라서 문자 리턴
///	@return	문자화 된 숫자.
string strutil::krstrncpy(
	const string &sSrc,			///< 한글이나 영문자.
	int nPoint,					///< 위치
	int nLength					///< 문자열 길이.
	)
{
	unsigned char cData;
	string	sConvert;
	string	sDes;
	string	sDesConvert;
	string 	sKor, sEng;
	int nCount = 0, nLengthCnt = 0, nSize = 0;

	if (sSrc == "") return "";
	if (nLength == 0) return "";

	// utf8은 euckr로 바꿔야지만 제대로 처리된다.
	if (e_nCurLang == KO_UTF8)
	{
		sConvert = CodeConvert(sSrc, "EUC-KR", "UTF-8");
	}
	else
		sConvert = sSrc;

	for( nCount=0; nCount < sConvert.size(); nCount++ )
	{
		sEng = ""; sKor = "";
		cData = sConvert[nSize];
		if ( cData < 0x0080 )
		{
			sEng = sConvert[nSize];
		}
		else
		{
			sKor = sConvert[nSize];
			sKor = sKor + sConvert[nSize+1];
		}
		
		if (nSize >= nPoint)
		{
			if (nSize - nPoint >= nLength) break;
			if (sEng != "")
				sDes = sDes + sEng;
			else
				sDes = sDes + sKor;
		}	
		
		if (nSize >= sConvert.size()) break;
		if (sEng != "") nSize++;
		if (sKor != "") nSize = nSize + 2;
	}

	if (sDes == "") return "";

	if (e_nCurLang == KO_UTF8)
		sDesConvert = CodeConvert(sDes, "UTF-8", "EUC-KR");
	else
		sDesConvert = sDes;
	return sDesConvert;
}

/// @brief	문자가 한글일 경우 Length 리턴 utf8인 경우 한글 사이즈처리 이상.
int strutil::krstrlen(
	const string &sSrc
	)
{
	string sConvert;
	
	if ((int)sSrc.size() == 0) return 0;
	
	// utf8은 euckr로 바꿔야지만 제대로 처리된다.
	if (e_nCurLang == KO_UTF8)
		sConvert = CodeConvert(sSrc, "EUC-KR", "UTF-8");
	else
		sConvert = sSrc;
	
	//LOG("sConvert [%s] [%d]", sSrc.c_str(), sConvert.size());
	return (int)sConvert.size();
}

/// @brief nLength 사이즈에 맞게 문자가 중간에 위치하게 만든다.
string strutil::strmid(
	int nLength,
	const string &sSrc
	)
{
	string sConvert;
	
	if (nLength == 0) return "";
	if (sSrc.size() == 0) return "";
	int nSrcStrLength = strutil::krstrlen(sSrc);
	if (nSrcStrLength >= nLength) return sSrc;	
	int nPos = (nLength - nSrcStrLength) / 2;
	sConvert.append(nPos, ' ');
	sConvert.append(sSrc);
	sConvert.append(nLength-(nPos+nSrcStrLength), ' ');
	return sConvert;
}
