#include "commandimp.h"
#include <sys/wait.h>
#include "strutil.h"
#include "qcd.h"
#include "TargzUtil.h"
#include "progress.h"

using namespace strutil;

extern int Copy(MLS::Selection &, std::string);
extern int Move(MLS::Selection &, std::string);

namespace MLS {

void CommandImp::UpdateConfig()
{
}

void CommandImp::SaveConfig()
{
}

void CommandImp::CopyClip()
{
	g_MainFrame->_tClipboard.Set(panel->GetSelection(), CLIP_COPY);
}

void CommandImp::CutClip()
{
	g_MainFrame->_tClipboard.Set(panel->GetSelection(), CLIP_CUT);
}

void CommandImp::PasteClip()
{
	ClipState state = g_MainFrame->_tClipboard.GetState();

	if (state == CLIP_NONE) return;

	if (state == CLIP_COPY)
	{
		if (panel->isZip() == false)
		{
			string sZipName = (g_MainFrame->_tClipboard.Get()).GetZipFile();
			vector<string> tFileList = (g_MainFrame->_tClipboard.Get()).GetData();
			if (tFileList.size() == 0) return;

			LOG("CLIP_COPY [%s]", sZipName.c_str());

			if (sZipName != "")
			{
				try
				{
					TargzUtil	tTargzUtil(sZipName, 0);

					string sView = gettext("Please wait !!! - Uncompress file [") + sZipName + "]";

					Progress 	tProgress(gettext("Wait"), sView.c_str());
					tProgress.Start((Counter*)&tTargzUtil, JOIN);
					tTargzUtil.SetCounter(0);

					tTargzUtil.Uncompress(tFileList, panel->_path);

					tTargzUtil.SetCounter(100);
					tProgress.SetRun(PTHREAD_END);
					tProgress.End();
				}
				catch(Exception& ex)
				{
					MsgBox(1, gettext("Error"), "%s", (char*)ex);
				}
			}
			else
			{
				::Copy(g_MainFrame->_tClipboard.Get(), panel->_path);
			}
		}
		else
		{
			// zip 파일 안의 내용에서 zip파일로는 복사 불가능.
			if (panel->isZip() == 1)
			{
				string sDir = (g_MainFrame->_tClipboard.Get()).GetBaseDir();
				vector<string> tFileList = (g_MainFrame->_tClipboard.Get()).GetData();
				
				if (tFileList.size() == 0) return;

				try
				{
					TargzUtil	tTargzUtil(panel->_Zip_Reader.GetZipPath(), panel->_sZipFile, 0);
					string sView = gettext("Please wait !!! - Uncompress file [") + panel->_sZipFile + "]";

					Progress 	tProgress(gettext("Wait"), sView.c_str());
					tProgress.Start((Counter*)&tTargzUtil, JOIN);
					tTargzUtil.SetCounter(0);

					tTargzUtil.SetWorkDir(sDir);
					tTargzUtil.Compress(tFileList, TAR_APPEND);

					tTargzUtil.SetCounter(100);
					tProgress.SetRun(PTHREAD_END);
					tProgress.End();

					panel->Refresh(true);
				}
				catch(Exception& ex)
				{
					MsgBox(1, gettext("Error"), "%s", (char*)ex);
				}
			}
		}
		Refresh();
	}
	else if (state == CLIP_CUT)
	{
		if (panel->isZip() == false)
		{
			string sZipName = (g_MainFrame->_tClipboard.Get()).GetZipFile();

			if (sZipName == "")
				::Move(g_MainFrame->_tClipboard.Get(), panel->_path);
		}
		Refresh();
	}
}

void CommandImp::Qcd()
{
	MLS::Qcd qcd;

	qcd.Do();
}

void CommandImp::ViewScreen( )
{
	g_MainFrame->Destroy();

	cbreak();
	noecho();

	getch();

	g_MainFrame->Init();
	g_MainFrame->Refresh();
}

///	@brief	명령어를 실행한다.
///	@param	cmd		명령어및 인자
///	@param	bPause	명령어를 실행시키고 종료시 잠시 정지시킬지 여부
/// @param	bFork	Fork할 것인지?
///	@return	명령어 실행 결과
int CommandImp::Run( const std::string &cmd, bool bPause, bool bBackground)
{
	//def_prog_mode();
	//endwin();

	g_MainFrame->Destroy();

	int status;

	if (bBackground)
	{
		string sCmd = cmd + " > /dev/null 2>&1 &";
		status = system(sCmd.c_str());
	}
	else
	{
		status = system(cmd.c_str());
	}

	if (bPause)
	{
		cbreak();
		noecho();

		cout << endl << gettext("Press any key to return Mls...") << '\r' << endl;
		getch();
	}
	g_MainFrame->Init();
	Refresh();

	return status;
}

};
