/**
 * @file	mainframe.cpp
 * @brief	mainframe source file
 * @author	la9527
 * @date	2004-08-22
 */
#include <sys/types.h>
#include <signal.h>

#include "define.h"
#include "mainframe.h"
#include "mlslocale.h"
#include "dialog.h"
#include "configure.h"

namespace MLS {

MainFrame *g_MainFrame;

MainFrame::MainFrame()
{
	_bSplit  = false;
	_nActive = 0;

	UpdateConfig();
}

MainFrame::~MainFrame()
{
	SaveConfig();
}

void MainFrame::UpdateConfig( )
{
	_Func[0].addEntry(config.GetValue("FuncF1"));
	_Func[1].addEntry(config.GetValue("FuncF2"));
	_Func[2].addEntry(config.GetValue("FuncF3"));
	_Func[3].addEntry(config.GetValue("FuncF4"));
	_Func[4].addEntry(config.GetValue("FuncF5"));
	_Func[5].addEntry(config.GetValue("FuncF6"));
	_Func[6].addEntry(config.GetValue("FuncF7"));
	_Func[7].addEntry(config.GetValue("FuncF8"));
	_Func[8].addEntry(config.GetValue("FuncF9"));
	_Func[9].addEntry(config.GetValue("FuncF10"));
	_Func[10].addEntry(config.GetValue("FuncF11"));
	_Func[11].addEntry(config.GetValue("FuncF12"));

	_bSplit = config.GetBool("SplitWindow");

}

void MainFrame::SaveConfig()
{
	config.SetBool("SplitWindow", _bSplit);
}


/// @brief		consol 초기화 작업.
bool MainFrame::Init()
{
	initscr();     // nCurses 시작
	start_color(); // 컬러 가능하게

	nonl();        // 엔터 처리?
	//raw();		   // Ctrl+C : Signal무시 가능하게
	cbreak();      // nCurses : 즉시 입력이 가능하도록 설정

	noecho();	   // nCurses : echo모드 해제
	keypad(stdscr, TRUE);  // keypad 활성화

	curs_set(0); // 커서를 보이지 않게 한다.

	// 색깔 쌍을 초기화 한다. 0 ~64개 까지
	for (int t=0; t<8; t++)
		for (int t2=0; t2<8; t2++)
	{
		if (t==0 && t2==0) continue;
		init_pair(t*8+t2, t, t2);
	}

	// curses 마우스 설정
	mousemask( 	BUTTON1_CLICKED | BUTTON2_CLICKED | BUTTON3_CLICKED | 
				BUTTON_SHIFT | BUTTON_CTRL, NULL);
	//mousemask(ALL_MOUSE_EVENTS, NULL);
	mouseinterval(10);
	_beforeEvent.x = 0; _beforeEvent.y = 0; // 더블클릭을 위한
	
	// line 설정
	Set_Line(e_nBoxLineCode);

	_bShell = false;
	setcol(g_Color.Default);
	bkgd(' ');

	// 커맨드를 초기화
	g_Command.Init();

	return true;
}

void MainFrame::Destroy()
{
	clear();
	refresh();

	// . keypad 비활성화
	keypad(stdscr, FALSE);  // . 라인단위로 입력이 이루어지도록 설정
	nocbreak();
	//noraw();
	curs_set(1); // 커서를 다시 보이게 한다.
	noecho();
	endwin();   // . nCurses 해제
}

void MainFrame::SetSplitWindow( )
{
	if (_bSplit)
	{
		_tPanel[0].SetWindow(_win1, (g_nCOLS-2)/2, g_nLINES-6);
		_tPanel[1].SetWindow(_win2, (g_nCOLS-1)/2, g_nLINES-6);
	}
	else
	{
		_tPanel[_nActive].SetWindow(_win0, g_nCOLS, g_nLINES-6);
	}
}

void MainFrame::Draw()
{
	////////////// Panel Draw
	if (_bSplit)
	{
		_tPanel[0].Draw(_nActive == 0);
		_tPanel[1].Draw(_nActive == 1);
	}
	else
	{
		_tPanel[_nActive].Draw(true);
	}

	int i;
	//////////////////. Function Draw

	setcol(g_Color.Func);
	move(0,0);
	hline(' ', g_nCOLS);
	
	// Funtion panel draw
	for (i=0; i<8; i++)
	{
		int pos = GetColumnIndex(g_nCOLS, 8, i);
		mvprintw(0, 2+pos, "%s", gettext(_Func[i+1].getName()));
	}

	setcol(g_Color.FuncA);
	mvprintw (0, 0, "F");
	for (i=0; i<8; i++)
	{
		int pos = GetColumnIndex(g_nCOLS, 8, i);
		mvprintw(0, 1+pos, "%d", i+2);
	}

	setcol(COLOR_BLACK, g_Color.Func.back);
	for (i=1; i<8;i++)
	{
		int pos = GetColumnIndex(g_nCOLS, 8, i);
		mvaddch(0, pos, VLINE);
	}

	////////////////// . PATH Draw
	setcol(g_Color.Stat);
	move(1, 0); hline(' ', g_nCOLS);
	if (_bSplit)
	{
		mvprintw(1, 0, "%s", _tPanel[0].GetPathView().c_str());
		mvprintw(1, g_nCOLS/2+1, "%s", _tPanel[1].GetPathView().c_str());
	}
	else
	{
		mvprintw(1, 0, "%s", _tPanel[_nActive].GetPathView().c_str());
	}

	//////// 상태선 그리기..
	_tPanel[_nActive].DrawStatus();

	//////// 2단일 경우. 중간의 꺾은선 출력.
	setcol(g_Color.Line);
	if (_bSplit)
	{
		for (int t=0; t<g_nLINES-6; t++)
		{
			move(3 + t, (g_nCOLS-2)/2);
			addch(VLINE);
			addch(VLINE);
		}

		move(2, (g_nCOLS-2)/2);
		addch(TTEE);
		addch(TTEE);

		move(g_nLINES-3, (g_nCOLS-2)/2);
		addch(BTEE);
		addch(BTEE);
	}
	////////////////////
	refresh();
}

void MainFrame::Refresh()
{
	g_nLINES = LINES;
	g_nCOLS = COLS;
			
	if (g_nLINES - 6 <= 0) return;
	if ((g_nCOLS-2)/2 <= 0) return;

	wresize(_win0, g_nLINES-6, g_nCOLS);
	wresize(_win1, g_nLINES-6, (g_nCOLS-2)/2);
	wresize(_win2, g_nLINES-6, (g_nCOLS-1)/2);

	mvwin(_win1, 3, 0);
	mvwin(_win2, 3, (g_nCOLS+2)/2);

	Draw();
	SetSplitWindow();
}

///	@brief 메인프레임 수행
/// @param bMcd Mcd실행 여부.
void MainFrame::Do( bool bMcd )
{
	_win0 = newwin(g_nLINES-6, g_nCOLS, 3, 0);
	_win1 = newwin(g_nLINES-6, (g_nCOLS-2)/2, 3, 0);
	_win2 = newwin(g_nLINES-6, (g_nCOLS-1)/2, 3, (g_nCOLS+2)/2);

	_bExit = false;
	
	_tPanel[0].Read(".");
	_tPanel[1].Read(".");

	g_Command.SetPanel(&_tPanel[0]);
	SetSplitWindow();
	refresh();

	ESCDELAY = 10;  // ESC 딜레이 줄이기.
	int nErrno = 0; // 에러 번호 체크
	bool bDraw = true;

	if (bMcd) g_Command.Execute("<mcd>");

	while(!_bExit)
	{
		// 패널 화면을 그린다.
		if (bDraw == true)	Draw();
		bDraw = true;
		
		// 쉘을 실행하면 계속 실행하게 한다.
		if (_bShell == true)
		{
			g_Command.Execute("<shell>");
			continue;
		}

		// 키를 입력 받는다.
		int nKey = getch();

		LOG("KEY [%d]", nKey);

		// 콘솔에 따라서 백그라운드로 남아 있는 것을 잡기 위한 방법
		if (nKey == -1)
		{
			nErrno++;
			if (nErrno == 100)
			{
				kill(getpid(), 3); // Signal Quit로 종료
			}
			continue;
		}
		else
		{
			nErrno = 0;
		}

		if (nKey == 3)
		{
			nKey = getch();
			g_Command.Execute("<copyclip>");
			continue;
		}

		// 리사이즈
		if (nKey == KEY_RESIZE)
		{
			Refresh();
			bDraw = false;
			continue;
		}

		// 마우스
		if (nKey == KEY_MOUSE)
		{
			MouseProc();
			continue;
		}

		// Searching..
		if (_tPanel[_nActive].SearchProcess(nKey))
		{
			continue;
		}

		// Function key
		if (KEY_F(1) <= nKey && nKey <=KEY_F(12))
		{
			if (_Func[nKey-KEY_F(1)].size()>=1)
			{
				g_Command.Execute(_Func[nKey-KEY_F(1)][0]);
			}
			continue;
		}

		// Execute
		if (nKey != 27)
		{
			string command = g_Keybind.GetCommand(nKey);
			g_Command.Execute(command);
		}
		else
		{
			// Alt + 문자는 이렇게 받는다.
			nodelay(stdscr, TRUE);
			nKey = getch();
			nodelay(stdscr, FALSE);

			if (nKey != ERR)
			{
				g_Command.Execute(g_Keybind.GetMetaCommand(nKey));
			}
			else
			{
				g_Command.Execute(g_Keybind.GetCommand(27));
			}
		}
	}
}

/// @brief	NextWindow
void MainFrame::NextWindow()
{
	if (_bSplit)
	{
		_nActive = (_nActive + 1) %2;
		g_Command.SetPanel(&_tPanel[_nActive]);

		// 디렉토리 이동
		chdir(_tPanel[_nActive].GetPathView().c_str());
	}
}

void MainFrame::RefreshPanel(bool bReload)
{
	if (_bSplit)
	{
		_tPanel[0].Refresh(bReload);
		_tPanel[1].Refresh(bReload);
	}
	else
	{
		_tPanel[_nActive].Refresh(bReload);
	}
	chdir(_tPanel[_nActive].GetPathView().c_str());
}

#define	MENU_XPOS				0		///< 화면상의 Menu Y 위치
#define	FUNCTIONKEY_TOTALCOUNT	8		///< 화면상의 Functionkey 개수
#define	FILELIST_MIN_POS		3		///< FileList 시작 y 위치.

/**
 * @brief	마우스 이벤트 처리
 */
void MainFrame::MouseProc()
{
	MEVENT event;
	getmouse(&event);

	LOG("Event [%ld]",event.bstate);
	
	// 마우스가 메뉴를 눌렸을 경우
	if(event.y == MENU_XPOS)
	{
		int functionkey;
		// 어느 메뉴를 눌렀는지 확인하여 실행
		functionkey = XPositionToFunctionkey(event.x);
		if(KEY_F(1) < functionkey && functionkey <= KEY_F(9))
		{
			g_Command.Execute(_Func[functionkey-KEY_F(1)][0]);
		}	
	}
	// 마우스가 파일List를 눌렀을경우
	else if(FILELIST_MIN_POS <= event.y)
	{
		int	panelIndex;
		// 해당 패널 구하기
		panelIndex = XPositionToFilePanel(event.x);

		// x, y 위치를 패널에 맞게 지정
		SetXYLocalToPanel(event.x, event.y, panelIndex);

		// 해당 패널 활성화.
		if(panelIndex != _nActive)
		{
			NextWindow();
		}

		if ((event.bstate & BUTTON1_CLICKED))
		{
			LOG("Event.x [%d] Event.y [%d] BeforeX [%d] BeforeY [%d]",
					event.x, event.y, _beforeEvent.x, _beforeEvent.y);

			if (event.x == _beforeEvent.x && event.y == _beforeEvent.y)
			{
				_tPanel[panelIndex].MouseProc(event);
				_tPanel[panelIndex].Key_Enter();
				_beforeEvent.x = 0; _beforeEvent.y = 0;
				return;
			}
			else
			{			
				_tPanel[panelIndex].MouseProc(event);
				_beforeEvent.x = event.x; _beforeEvent.y = event.y;
			}
		}

		if ((event.bstate & BUTTON3_CLICKED))
		{
			_tPanel[panelIndex].MouseProc(event);
			_tPanel[panelIndex].ToggleSelect();
		}
	}
	return;
}

/**
 * @brief	화면상의 X position을 가지고 Function key을 얻는다
 * @param xPosition 화면상의 X position
 * @return	Function key (KEY_F(2) ~ KEY_F(8))
 **/
int MainFrame::XPositionToFunctionkey( const int xPosition )
{
	int index = GetIndex(g_nCOLS, FUNCTIONKEY_TOTALCOUNT, xPosition);
	int x     = GetColumnIndex(g_nCOLS, FUNCTIONKEY_TOTALCOUNT, index);
	
	if (x && xPosition == x) return -1;
	return KEY_F(index + 2);
}

/// @brief	window split switching Function.
void MainFrame::SwitchSplit()
{
	_bSplit = !_bSplit;
}

/// @brief	x 값으로 해당 패널을 찾는 함수.
/// @param x 화면 X position
/// @return	해당 패널 index.
int MainFrame::XPositionToFilePanel(const int x)
{
	if(!_bSplit || x < g_nCOLS/2)
		return 0;

	return 1;
}

/// @brief	패널에 맞게 X, Y위치를 변경한다.
/// @param	x			[OUT]	x위치.
/// @param	y			[OUT]	y위치.
/// @param	panelindex	패널 위치. 
void MainFrame::SetXYLocalToPanel(int & x, int & y, const int panelindex)
{
	x = x - panelindex * ((int)g_nCOLS/2);

	//	두번째 패널이면 가운데 구분선을 빼어 준다.
	x = panelindex == 1 ? x-1 : x;
	y = y - FILELIST_MIN_POS;
}

}  // namespace
