/**
 * @file	mainframe.h
 * @brief	mainframe header file
 * @author	la9527
 * @date	2004-08-22
 */
#ifndef MLSMAINFRAME_H
#define MLSMAINFRAME_H

#include "define.h"
#include "panel.h"
#include "DirMcd.h"
#include "command.h"
#include "clipboard.h"
#include "colorset.h"
#include "configure.h"

namespace MLS {
/**
	@brief	MainFrame 클래스

	Panel들을 관리한다.
*/
					
class MainFrame : public IConfigurable
{
private:
	Panel   	_tPanel[2];
	
	Menu    	_tMenu;
	ClipBoard   _tClipboard;
	
	bool        _bExit;
	bool		_bShell;
	bool		_bSplit;   ///< 창나누기인가 아닌가.
	int			_nActive;  ///< Active한 창을 가리킨다 0, 1
	MEVENT 		_beforeEvent; ///< 더블클릭 대체 방법으로 쓰임
	
	WINDOW *	_win0;     ///< 전체 윈도우
	WINDOW *	_win1;
	WINDOW *	_win2;
	
	//  configuration
	Action      _Func[12];

	void		MouseProc();
	int			XPositionToFunctionkey(const int xPosition);
	int			XPositionToFilePanel(const int x);
	void		SetXYLocalToPanel(int & x, int & y, const int panelindex);

public:
	MainFrame();
	~MainFrame();
	
	bool    Init();
	void    Destroy();
	
	void 	Do(bool bMcd = false);
	void    Draw();
	void	SetSplitWindow();
	void    Exit() { _bExit = true; }
	void	ShellExe(bool Shell) { _bShell = Shell; }
    void	NextWindow();
	void	Refresh();
	void    RefreshPanel(bool bReload = false);
	void	ReloadMenu() { _tMenu.Create();	}
	
	void	SwitchSplit();
	bool	IsSplit()	{ return _bSplit; }

	void    UpdateConfig();
	void    SaveConfig();

	/// @brief		Func키의 Cmd를 가져온다.(Menu에서 쓰임)
	int		GetFuncCmd(string sStr)
	{
		for (int n=0; n<12; n++)
			if (_Func[n][0]==sStr) return n+1;
		return 0;
	}

	int		g_ng_nCOLS;
	int		g_ng_nLINES;
	
	friend class CommandImp;
	friend class Panel;
};

extern MainFrame *g_MainFrame;
};

#endif
